@@ -45,9 +45,9 @@ dbgamma<-function(x,delta.plus,gamma.plus,delta.minus,gamma.minus){
 }
 
 
-## (Multivariate) Normal gamma
+## (Multivariate) Variance gamma
 
-rngamma <- function(x,lambda,alpha,beta,mu,Lambda){
+rvgamma <- function(x,lambda,alpha,beta,mu,Lambda){
   ## Error check
   if(length(mu)!=length(beta)){
     stop("Error: wrong input dimension.")
@@ -82,10 +82,12 @@ rngamma <- function(x,lambda,alpha,beta,mu,Lambda){
     if( nrow(Lambda)!=ncol(Lambda)){
       stop("Lambda must be a square matrix.")
     }
+    if(sum((Lambda-t(Lambda))*(Lambda-t(Lambda)))!=0){
+      stop("Lambda must be a symmetric matrix")
+    }
     if( nrow(Lambda)!=length(beta)){
       stop("Dimension of Lambda and beta must be equal.")
     }
-    
     if( min(eigen(Lambda)$value) <= 10^(-15) ){
       stop("Lambda must be positive definite.")
     }
@@ -100,7 +102,7 @@ rngamma <- function(x,lambda,alpha,beta,mu,Lambda){
     if( alpha <= 0 )
       stop("alpha must be positive.")
     if( tmp <=0)
-      stop("alpha^2 - t(beta) %*% Lambda %*% must be positive.")
+      stop("alpha^2 - t(beta) %*% Lambda %*% beta must be positive.")
     
     tau <- rgamma(x,lambda,tmp/2)
     eta <- rnorm(x*length(beta))
@@ -114,7 +116,7 @@ rngamma <- function(x,lambda,alpha,beta,mu,Lambda){
 }
 
 
-dngamma <- function(x,lambda,alpha,beta,mu,Lambda){
+dvgamma <- function(x,lambda,alpha,beta,mu,Lambda){
   ## Error check
   if(length(lambda)!=1||length(alpha)!=1)
     stop("alpha and lambda must be positive reals.")
@@ -141,6 +143,9 @@ dngamma <- function(x,lambda,alpha,beta,mu,Lambda){
     if( nrow(Lambda)!=ncol(Lambda)){
       stop("Lambda must be a square matrix.")
     }
+    if(sum((Lambda-t(Lambda))*(Lambda-t(Lambda)))!=0){
+      stop("Lambda must be a symmetric matrix")
+    }
     if( nrow(Lambda)!=length(beta)){
       stop("Dimension of Lambda and beta must be equal.")
     }
@@ -154,7 +159,7 @@ dngamma <- function(x,lambda,alpha,beta,mu,Lambda){
 
     tmp <- as.numeric(alpha^2 - t(beta) %*% Lambda %*% beta)
     if( tmp <=0)
-      stop("alpha^2 - t(beta) %*% Lambda %*% must be positive.")
+      stop("alpha^2 - t(beta) %*% Lambda %*% beta must be positive.")
     Lambdainv<-solve(Lambda)
     dens<- exp(t(beta)%*%(x-mu))*(alpha^2-t(beta)%*%Lambda%*%beta)^(lambda)*besselK(alpha*sqrt(t(x-mu)%*%Lambdainv%*%(x-mu)),lambda-nrow(Lambda)/2)*sqrt(t(x-mu)%*%Lambdainv%*%(x-mu))^{lambda-nrow(Lambda)/2}/(gamma(lambda)*pi^{nrow(Lambda)/2}*2^{nrow(Lambda)/2+lambda-1}*alpha^{lambda-nrow(Lambda)/2})
     dens
@@ -202,6 +207,13 @@ rNIG <- function(x,alpha,beta,delta,mu,Lambda){
   if(length(mu)!=length(beta)){
     stop("Error: wrong input dimension.")
   }
+  if(length(alpha)>1||length(delta)>1)
+    stop("alpha and delta must be positive reals.")
+  if( alpha < 0 )
+    stop("alpha must be nonnegative.")
+  if( delta <= 0 )
+    stop("delta must be positive.")
+  
   if(missing(Lambda))
    Lambda <- NA
 
@@ -224,7 +236,15 @@ rNIG <- function(x,alpha,beta,delta,mu,Lambda){
     return(X)
     
   }else{  ## multivariate case
-  	
+    if( nrow(Lambda)!=ncol(Lambda)){
+      stop("Lambda must be a square matrix.")
+    }
+    if(sum((Lambda-t(Lambda))*(Lambda-t(Lambda)))!=0){
+      stop("Lambda must be a symmetric matrix")
+    }
+    if( nrow(Lambda)!=length(beta)){
+      stop("Dimension of Lambda and beta must be equal.")
+    }
 	if( min(eigen(Lambda)$value) <= 10^(-15) ){
       stop("Lambda must be positive definite.")
     }
@@ -278,6 +298,9 @@ dNIG <- function(x,alpha,beta,delta,mu,Lambda){
     if( nrow(Lambda)!=ncol(Lambda)){
       stop("Lambda must be a square matrix.")
     }
+    if(sum((Lambda-t(Lambda))*(Lambda-t(Lambda)))!=0){
+      stop("Lambda must be a symmetric matrix")
+    }
     if( nrow(Lambda)!=length(beta)){
       stop("Dimension of Lambda and beta must be equal.")
     }
@@ -366,28 +389,73 @@ rstable <- function(x,alpha,beta,sigma,gamma){
 
 
 ## Positive exponentially tempered stable (AR method)
-## ## This must be re-coded later!! (temporarily, rather inefficient)
-rpts <- function(x,al,a,b) {
-    if( al <= 0 | al>= 1 )
-    stop("al must lie in (0,1).")
-    if( a <= 0 )
+rpts<-function(x,alpha,a,b){
+  if( alpha <= 0 | alpha>= 1 )
+    stop("alpha must lie in (0,1).")
+  if( a <= 0 )
     stop("a must be positive value.")
-    if( b <= 0 )
+  if( b <= 0 )
     stop("b must be positive value.")
+  ar<-exp(a*gamma(-alpha)*b^(alpha))
+  if(ar <= 0.1)
+    stop("Acceptance rate is too small.")
+  else
+    .C("rpts",as.integer(x),as.double(alpha),as.double(a),as.double(b),rn=double(length=x))$rn}
 
-	sig <- (-a*gamma(-al)*cos(pi*al/2))^(1/al)
-	y <- c()
-	i <- 1
-	while (i <= x) {
-		u <- runif(1)
-		v <- rstable(1,al,1,sig,0)
-		w <- exp(-b*v)
-		if (u < w ){
-			y <- append(y,v)
-			i <- i+1
-		}
-	}
-	return(y)
+
+## Normal tempered stable 
+rnts<-function(x,alpha,a,b,beta,mu,Lambda){
+  ## Error check
+  if(length(mu)!=length(beta)){
+    stop("Error: wrong input dimension.")
+  }
+  if(missing(Lambda))
+    Lambda <- NA
+  if( alpha <= 0 || alpha > 2 ){
+    stop("The index alpha must take values in (0,2].")
+  }
+  if( a <= 0 )
+    stop("a must be positive value.")
+  if( b <= 0 )
+    stop("b must be positive value.")
+  
+  if(is.na(Lambda)){
+    ## univariate case
+    if(length(mu)!=1 || length(beta)!=1){
+      stop("Error: wrong input dimension.")
+    }
+    
+    tau <- rpts(x,alpha,a,b)
+    eta <- rnorm(x)
+    ##  z <- mu + beta * tau * Lambda + sqrt(tau * Lambda) * eta
+    z <- mu + beta * tau + sqrt(tau) * eta
+    X <- z
+    return(X)
+    
+  }else{ ## multivariate case
+    if( nrow(Lambda)!=ncol(Lambda)){
+      stop("Lambda must be a square matrix.")
+    }
+    if( nrow(Lambda)!=length(beta)){
+      stop("Dimension of Lambda and beta must be equal.")
+    }
+    if(nrow(Lambda)!=length(mu)){
+      stop("Dimension of Lambda and mu must be equal.")
+    }
+    if( min(eigen(Lambda)$value) <= 10^(-15) ){
+      stop("Lambda must be positive definite.")
+    }
+    if( det(Lambda) > 1+10^(-15) || det(Lambda) < 1-10^(-15) ){
+      stop("The determinant of Lambda must be 1.")
+    }
+    tau<-rpts(x,alpha,a,b)
+    eta <- rnorm(x*length(beta))
+    sqrt.L <- svd(Lambda)
+    sqrt.L <- sqrt.L$u %*% diag(sqrt(sqrt.L$d)) %*% t(sqrt.L$v)
+    
+    z <- mu + t(matrix(rep(tau,length(beta)),x,length(beta))) * matrix(rep(Lambda %*% beta,x),length(beta),x)+t(matrix(rep(sqrt(tau),length(beta)),x,length(beta))) * (sqrt.L %*% t(matrix(eta,x,length(beta))))
+    z<-mu+ t(matrix(rep(tau,length(beta)),x,length(beta))) * matrix(rep(Lambda %*% beta,x),length(beta),x)+t(matrix(rep(sqrt(tau),length(beta)),x,length(beta))) * (sqrt.L %*% t(matrix(eta,x,length(beta))))
+    X <- z
+    return(X)
+  }
 }
-
-
